import unittest
import os

from longtang.actors import testutils
from longtang.actors.filesystem import filemanager, messages, domain
from longtang.common import domain as common_domain
from hamcrest import *

class TestFileManagerActor(unittest.TestCase):

	@classmethod
	def setUpClass(cls):
		global base_test_dir
		global base_io_target_dir

		base_test_dir = testutils.create_tmp_dir(suffix='_file_management_test')
		base_io_target_dir = testutils.create_tmp_dir(suffix='_file_management_test_io_target')

	@classmethod
	def tearDownClass(cls):
		testutils.remove_tmp_dir(base_test_dir)
		testutils.remove_tmp_dir(base_io_target_dir)

	def setUp(self):

		global source_file

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = testutils.copy_to_tmp(source=os.path.join(data_dir, 'rename.mp3'),suffix='.tmp.mp3', dir=base_test_dir)

	def tearDown(self):
		testutils.remove_dir_content(base_test_dir)	

	def test_rename_file_from_metadata_message(self):

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('Dummy Artist').title('Dummy Title').album('Dummy Album').track_number('7').build()

		file_tester = testutils.TestActorBuilder().withType(filemanager.FileManagerActor).build()

		file_tester.tell(messages.RenameFileFromMetadata(source_file, metadata, 'tracking-id'))

		assert_that(file_tester.inspector().num_instances(messages.FileSuccessfullyRenamed), is_(equal_to(1)), 'Total amount of FileSuccessfullyRenamed messages received is wrong')

		message = file_tester.inspector().get_first(messages.FileSuccessfullyRenamed)

		assert_that(message.source_file(), is_(not_none()), 'Source file is empty')
		assert_that(message.new_file(), is_(not_none()), 'New file is empty')
		assert_that(message.metadata(), is_(not_none()), 'File metadata information is empty')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')

	def test_rename_file_filename_structure(self):

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('Dummy Artist').title('Dummy Title').album('Dummy Album').track_number('7').build()

		file_tester = testutils.TestActorBuilder().withType(filemanager.FileManagerActor).build()

		file_tester.tell(messages.RenameFileFromMetadata(source_file, metadata, 'tracking-id'))

		assert_that(file_tester.inspector().num_instances(messages.FileSuccessfullyRenamed), is_(equal_to(1)), 'Total amount of FileSuccessfullyRenamed messages received is wrong')

		message = file_tester.inspector().get_first(messages.FileSuccessfullyRenamed)

		filepath, new_full_name = os.path.split(message.new_file())

		assert_that(os.path.isfile(message.new_file()), is_(equal_to(True)), 'File does not exist')
		assert_that(new_full_name, is_(equal_to('07 - Dummy Title.mp3')),'Filename is not right!')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')

	def test_copy_file(self):

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('Dummy Artist').title('Dummy Title').album('Dummy Album').track_number('7').build()

		file_tester = testutils.TestActorBuilder().withType(filemanager.FileManagerActor).build()

		file_tester.tell(messages.CopyFileTo(source_file, metadata, base_io_target_dir, 'tracking-id'))

		assert_that(file_tester.inspector().num_instances(messages.FileSuccessfullyCopied), is_(equal_to(1)), 'Total amount of FileSuccessfullyCopied messages received is wrong')		

		message = file_tester.inspector().get_first(messages.FileSuccessfullyCopied)

		assert_that(message.source_file(), is_(not_none()), 'Source file is empty')
		assert_that(message.copied_file(), is_(not_none()), 'Copied file path is empty')
		assert_that(message.metadata(), is_(not_none()), 'File metadata information is empty')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')
		assert_that(os.path.isfile(message.copied_file()), is_(equal_to(True)), 'Copied file does not exist')

	def test_copy_file_failure(self):

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('Dummy Artist').title('Dummy Title').album('Dummy Album').track_number('7').build()

		file_tester = testutils.TestActorBuilder().withType(filemanager.FileManagerActor).build()

		file_tester.tell(messages.CopyFileTo('/dummy_path.mp3', metadata, base_io_target_dir, 'tracking-id'))

		assert_that(file_tester.inspector().num_instances(messages.CopyFileToFailed), is_(equal_to(1)), 'Total amount of CopyFileToFailed messages received is wrong')

		message = file_tester.inspector().get_first(messages.CopyFileToFailed)

		assert_that(message.source_file(), is_(not_none()), 'Source file is empty')
		assert_that(message.metadata(), is_(not_none()), 'File metadata information is empty')
		assert_that(message.copy_to_path(), is_(not_none()), 'Copy path to information is empty')
		assert_that(message.reason(), is_(not_none()), 'Reason information is empty')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')

	def test_rename_file_from_metadata_message_failure(self):

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('Dummy Artist').title('Dummy Title').album('Dummy Album').track_number('7').build()

		file_tester = testutils.TestActorBuilder().withType(filemanager.FileManagerActor).build()

		file_tester.tell(messages.RenameFileFromMetadata('/dummy_path.mp3', metadata, 'tracking-id'))

		assert_that(file_tester.inspector().num_instances(messages.RenameFileFromMetadataFailed), is_(equal_to(1)), 'Total amount of RenameFileFromMetadataFailure messages received is wrong')

		message = file_tester.inspector().get_first(messages.RenameFileFromMetadataFailed)

		assert_that(message.source_file(), is_(not_none()), 'Source file is empty')
		assert_that(message.metadata(), is_(not_none()), 'File metadata information is empty')
		assert_that(message.reason(), is_(not_none()), 'Reason information is empty')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')


class LongTangConfiguration():
	
	def __init__(self):
		self.__read_from=None
		self.__generate_at=None
		self.__override_tags=None
		self.__offline_mode=None

	@property
	def read_from(self):
		return self.__read_from

	@read_from.setter
	def read_from(self,value):
		self.__read_from=value

	@property
	def generate_at(self):
		return self.__generate_at

	@generate_at.setter
	def generate_at(self,value):
		self.__generate_at=value

	@property
	def override_tags(self):
		return self.__override_tags

	@override_tags.setter
	def override_tags(self, value):
		self.__override_tags = value

	@property
	def offline_mode(self):
		return self.__override_tags

	@offline_mode.setter
	def offline_mode(self, value):
		self.__offline_mode = value


class LongTangConfigurationBuilder():

	def __init__(self):
		self.__read_from=None
		self.__generate_at=None
		self.__override_tags=False
		self.__offline_mode=False

	def read_from(self, path):
		self.__read_from=path
		return self

	def generate_at(self, path):
		self.__generate_at=path
		return self

	def override_tags(self, value):
		self.__override_tags=value
		return self

	def offline_mode(self, value):
		self.__offline_mode=value
		return self

	def build(self):
		config = LongTangConfiguration()

		#TODO We have to check whether the minimum values have been informed
		config.read_from=self.__read_from
		config.generate_at=self.__generate_at
		config.override_tags=self.__override_tags
		config.offline_mode=self.__offline_mode

		return config


#https://github.com/hamcrest/PyHamcrest
#http://packages.python.org/PyHamcrest/

import gevent
import unittest
import os

from hamcrest import *
from longtang.actors.supervisors.flowconductor import flowconductor, messages, factory
from longtang.actors import testutils
from longtang.system import system, exceptions
from longtang.config import configuration
from longtang.actors.tracking import tracking, facade
from longtang.actors.media.id3tag import domain

class TestFlowConductorActor(unittest.TestCase):

	def setUp(cls):
		global base_from_test_dir
		global base_to_test_dir

		base_from_test_dir = testutils.create_tmp_dir(suffix='flowconductor_test_from')
		base_to_test_dir = testutils.create_tmp_dir(suffix='flowconductor_test_to')

	def tearDown(cls):
		testutils.remove_tmp_dir(base_from_test_dir)
		testutils.remove_tmp_dir(base_to_test_dir)

	def test_creation(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		config_builder = configuration.LongTangConfigurationBuilder()
		config = config_builder.read_from(base_from_test_dir).generate_at(base_to_test_dir).build()

		actor_system = system.ActorSystem()
		main_actor = actor_system.with_factory(factory.FlowConductorSupervisorFactory(config),'flowconductor-test-actor')
		
		try:		
			assert_that(actor_system.find_by_id('flowconductor-test-actor'), is_not(None), 'Main actor does not exist within system')

			assert_that(actor_system.find_by_id('id3taghandler-actor'), is_not(None), 'Id3tag handler poller actor does not exist within system')
			assert_that(actor_system.find_by_id('filepackaging-actor'), is_not(None), 'File packaging poller actor does not exist within system')

			assert_that(actor_system.find_by_id('id3taghandler-actor').parent(), is_(equal_to(actor_system.find_by_id('flowconductor-test-actor'))),'Parent actor is not correctly assigned')
			assert_that(actor_system.find_by_id('filepackaging-actor').parent(), is_(equal_to(actor_system.find_by_id('flowconductor-test-actor'))),'Parent actor is not correctly assigned')

		except exceptions.ActorNotFound as e:
			self.fail(str(e))

		actor_system.shutdown()

	def test_single_file_processing_with_tags(self):
		
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		source_file_with_id3tag = testutils.copy_to_tmp(source=os.path.join(data_dir, 'with_id3taginfo.mp3'),suffix='.tmp.mp3',dir=base_from_test_dir)

		config_builder = configuration.LongTangConfigurationBuilder()
		config = config_builder.read_from(base_from_test_dir).generate_at(base_to_test_dir).build()

		flowconductor_actor = testutils.TestActorBuilder().withFactory(factory.FlowConductorSupervisorFactory(config))\
															.withId('flowconductor-test-actor')\
															.terminationMoratorium(10)\
															.terminate_system(False)\
															.terminationType(messages.MediaFileHasBeenProcessed).build()

		tracking_facade = facade.MediaTrackingFacade.create_within(flowconductor_actor.system())
		tracking_id = tracking_facade.create_tracking(source_file_with_id3tag)

		flowconductor_actor.tell(messages.MediaFileAvailable(source_file_with_id3tag, tracking_id))

		assert_that(flowconductor_actor.inspector().num_instances(messages.MediaFileHasBeenProcessed), is_(equal_to(1)), 'Total amount of MediaFileHasBeenProcessed messages received is wrong')

		message = flowconductor_actor.inspector().get_first(messages.MediaFileHasBeenProcessed)		

		#location
		full_dir_path=os.path.join(base_to_test_dir, 'Bon Jovi', 'New Jersey')
		full_file_path=os.path.join(full_dir_path,'08 - Ride Cowboy Ride.mp3')

		assert_that(os.path.isdir(full_dir_path), is_(equal_to(True)), 'Artist or album directory does not exist or full name is not right')
		assert_that(os.path.isfile(full_file_path), is_(equal_to(True)),'Media file in target path does not exist')

		#Tracking information
		assert_that(message.tracking(), is_(equal_to(tracking_id)), 'Tracking id does not match')

		tracking_info = tracking_facade.lookup(tracking_id)

		assert_that(tracking_info.mediafile.destinationpath, is_(equal_to(full_file_path)),'Filepath information on received message is wrong')

		#id3tag
		reader = domain.ID3TagReaderFactory.createFromType(full_file_path)

		assert_that(reader.artist(), is_(equal_to('Bon Jovi')), 'id3tag content for artist does not match')
		assert_that(reader.title(), is_(equal_to('Ride Cowboy Ride')), 'id3tag content for title does not match')
		assert_that(reader.album(), is_(equal_to('New Jersey')), 'id3tag content for album does not match')
		assert_that(reader.track_number(), is_(equal_to('8')), 'id3tag content for track number does not match')

	def test_single_file_processing_invalid_metadata(self):
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		source_file_with_id3tag = testutils.copy_to_tmp(source=os.path.join(data_dir, 'invalid_metadata.mp3'),suffix='.tmp.mp3',dir=base_from_test_dir)		

		config_builder = configuration.LongTangConfigurationBuilder()
		config = config_builder.read_from(base_from_test_dir).generate_at(base_to_test_dir).build()

		flowconductor_actor = testutils.TestActorBuilder().withFactory(factory.FlowConductorSupervisorFactory(config))\
															.withId('flowconductor-test-actor')\
															.terminationMoratorium(10)\
															.terminate_system(True)\
															.terminationType(messages.MediaFileProcessingFailed).build()

		tracking_facade = facade.MediaTrackingFacade.create_within(flowconductor_actor.system())
		tracking_id = tracking_facade.create_tracking(source_file_with_id3tag)

		flowconductor_actor.tell(messages.MediaFileAvailable(source_file_with_id3tag, tracking_id))

		assert_that(flowconductor_actor.inspector().num_instances(messages.MediaFileProcessingFailed), is_(equal_to(1)), 'Total amount of MediaFileProcessingFailed messages received is wrong')

		message = flowconductor_actor.inspector().get_first(messages.MediaFileProcessingFailed)

		assert_that(message.tracking(), is_(equal_to(tracking_id)), 'Tracking id does not match')

	def test_single_file_processing_nonexisting_path(self):

		config_builder = configuration.LongTangConfigurationBuilder()
		config = config_builder.read_from(base_from_test_dir).generate_at(base_to_test_dir).build()

		flowconductor_actor = testutils.TestActorBuilder().withFactory(factory.FlowConductorSupervisorFactory(config))\
															.withId('flowconductor-test-actor')\
															.terminationMoratorium(10)\
															.terminate_system(True)\
															.terminationType(messages.MediaFileProcessingFailed).build()

		tracking_facade = facade.MediaTrackingFacade.create_within(flowconductor_actor.system())
		tracking_id = tracking_facade.create_tracking('/dummy.mp3')

		flowconductor_actor.tell(messages.MediaFileAvailable('/dummy.mp3', tracking_id))

		assert_that(flowconductor_actor.inspector().num_instances(messages.MediaFileProcessingFailed), is_(equal_to(1)), 'Total amount of MediaFileProcessingFailed messages received is wrong')

		message = flowconductor_actor.inspector().get_first(messages.MediaFileProcessingFailed)

		assert_that(message.tracking(), is_(equal_to(tracking_id)), 'Tracking id does not match')

	def test_single_file_processing_nonexisting_destinationpath(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		source_file_with_id3tag = testutils.copy_to_tmp(source=os.path.join(data_dir, 'with_id3taginfo.mp3'),suffix='.tmp.mp3',dir=base_from_test_dir)

		config_builder = configuration.LongTangConfigurationBuilder()
		config = config_builder.read_from(base_from_test_dir).generate_at('/dummy').build()

		flowconductor_actor = testutils.TestActorBuilder().withFactory(factory.FlowConductorSupervisorFactory(config))\
															.withId('flowconductor-test-actor')\
															.terminationMoratorium(10)\
															.terminate_system(False)\
															.terminationType(messages.MediaFileProcessingFailed).build()

		tracking_facade = facade.MediaTrackingFacade.create_within(flowconductor_actor.system())
		tracking_id = tracking_facade.create_tracking(source_file_with_id3tag)

		flowconductor_actor.tell(messages.MediaFileAvailable(source_file_with_id3tag, tracking_id))

		assert_that(flowconductor_actor.inspector().num_instances(messages.MediaFileProcessingFailed), is_(equal_to(1)), 'Total amount of MediaFileProcessingFailed messages received is wrong')

		message = flowconductor_actor.inspector().get_first(messages.MediaFileProcessingFailed)

		assert_that(message.tracking(), is_(equal_to(tracking_id)), 'Tracking id does not match')	

if __name__ == '__main__':
	unittest.main()    


import domain

from longtang.actors import actors

class ActorRegistrationStrategy:

	def create_instance(self, unique_id, system_ref):
		raise NotImplemented()

	def create_ref(self, actor_instance, parent):
		raise NotImplemented()

	def signup(self, actor_ref, system_ref):
		raise NotImplemented()

	def start(self, actor_ref):
		raise NotImplemented()

class ActorTypeRegistrationStrategy(ActorRegistrationStrategy):
	def __init__(self, actor_type):
		self.__actor_type = actor_type

	def create_instance(self, unique_id, system_ref):
		return self.__actor_type(system_ref, unique_id)		

	def create_ref(self, actor_instance, parent):
		return domain.ActorRef(actor_instance, parent)

	def signup(self, actor_ref, system_ref):
		system_ref.register_actor(actor_ref)
		actor_ref.delegate_setup()
		system_ref.update_actor(actor_ref)

	def start(self, actor_ref):
		actor_ref.kickoff()

class ActorFactoryRegistrationStrategy(ActorRegistrationStrategy):
	def __init__(self, actor_factory):
		self.__actor_factory = actor_factory

	def create_instance(self, unique_id, system_ref):
		return self.__actor_factory.create(unique_id, system_ref)

	def create_ref(self, actor_instance, parent):
		return domain.ActorRef(actor_instance, parent)

	def signup(self, actor_ref, system_ref):
		system_ref.register_actor(actor_ref)
		actor_ref.delegate_setup()
		system_ref.update_actor(actor_ref)

	def start(self, actor_ref):
		actor_ref.kickoff()

class ActorSystemRegistration():

	def __init__(self, strategy, system_ref):
		self.__registration_strategy = strategy
		self.__system_ref = system_ref

	def add(self, unique_id, parent):
		actor_instance = self.__registration_strategy.create_instance(unique_id, self.__system_ref)
		actor_ref = self.__registration_strategy.create_ref(actor_instance, parent)

		self.__registration_strategy.signup(actor_ref, self.__system_ref)
		self.__registration_strategy.start(actor_ref)

		return actor_ref


# This file is part of longtang.
# Copyright 2013, Guillermo Szeliga.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.

import os
from setuptools import setup, find_packages

setup(name='longtang',
      version='1.0.0',
      description=
        'Audio collection id3tag tagger and organizer',
      author='Guillermo Szeliga',
      author_email='gszeliga@gmail.com',
      url='https://bitbucket.org/gszeliga/longtang',
      download_url='',
      license='MIT',
      platforms='ALL',
      long_description=open('README.md', 'rt').read(),
      install_requires = ['pyacoustid','gevent','nose','PyHamcrest', 'mutagen','argparse'],
      packages=find_packages(),
      scripts=['bin/longtang'],
      test_suite = 'nose.collector',
      #test_suite = "longtang.test.test_all",
      classifiers=[
		  'Development Status :: 6 - Mature',
		  'Environment :: Console',
		  'Intended Audience :: End Users/Desktop',
		  'Operating System :: Unix',
		  'Programming Language :: Python :: 2.7',
		  'Topic :: Multimedia :: Sound/Audio',
          'Topic :: Multimedia :: Sound/Audio :: Conversion',
          'Topic :: Multimedia :: Sound/Audio :: Editors',
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Topic :: Utilities'
      ],
)



class SystemProperties():

	def __init__(self):
		self.__verbosity=None

	@property
	def verbosity(self):
		return self.__verbosity

	@verbosity.setter
	def verbosity(self, level):
		self.__verbosity=level


class SystemPropertiesBuilder():

	def __init__(self):
		self.__verbosity=VerbosityLevel.DEBUG

	def verbosity(self, level):
		self.__verbosity=level
		return self

	def build(self):

		props = SystemProperties()

		props.verbosity=self.__verbosity

		return props

class VerbosityLevel:
    ERROR = 'ERROR'
    DEBUG = 'DEBUG'
    INFO = 'INFO'
    WARN = 'WARNING'


import gevent
import domain
import strategies
import exceptions
import deadletters
import syslogger
import properties

class ActorSystem():

	def __init__(self, props=None):
		self.__props=props

		if self.__props is None:
			self.__props = properties.SystemPropertiesBuilder().build() #grab defaults

		self.__actors_ref={}
		self.__actors_hierarchy={}
		self.__dead_letters_manager = deadletters.DeadLettersManager()
		self.__dead_letters_actor_ref = self.__dead_letters_manager.register_within_system(self)
		self.__syslogger = syslogger.SystemLoggerFactory.createIfAbsent(self, self.__props.verbosity)

	def __resolve_parent(self, parent):
		if parent is not None:
			if not isinstance(parent, deadletters.DeadLetters):
				return parent
			else:
				return None
		else:
			return self.__dead_letters_actor_ref

	def _register_actor_ref(self, actor_ref):
		self.__actors_ref[actor_ref.unique_id()] = actor_ref

		if actor_ref.parent() is not deadletters.DeadLetters.ROOT_PARENT:
			if actor_ref.parent().unique_id() in self.__actors_hierarchy.keys():
				self.__actors_hierarchy[actor_ref.parent().unique_id()].append(actor_ref.unique_id())
			else:
				self.__actors_hierarchy[actor_ref.parent().unique_id()]=[actor_ref.unique_id()]
		else:
			#We've just received dead-letters
			self.__actors_hierarchy[actor_ref.unique_id()]=[]

	def _update_actor_ref(self, actor_ref):
		self.__actors_ref[actor_ref.unique_id()] = actor_ref		

	def from_type(self, actor_type, unique_id, parent=None):
		strategy = strategies.ActorTypeRegistrationStrategy(actor_type)
		return self.__add_actor(strategy, unique_id, parent)

	def with_factory(self, factory, unique_id, parent=None):
		strategy = strategies.ActorFactoryRegistrationStrategy(factory)
		return self.__add_actor(strategy, unique_id, parent)

	def shutdown(self):
		xx = [actor._instance() for actor in self.__actors_ref.itervalues()]

		#TODO Tell dead_letters to finish
		#TODO Tell logger to finish

		gevent.killall(xx,gevent.GreenletExit,True)

	def wait_for(self, actor_ref):
		gevent.joinall([actor_ref._instance()])

	def __add_actor(self, strategy, unique_id, parent):
		actor_reg = strategies.ActorSystemRegistration(strategy, domain.WritableSystemRef(self))
		actor_ref = actor_reg.add(unique_id, self.__resolve_parent(parent))

		try:
			self.active_logger().debug(None, "{0}....Created".format(actor_ref), None)
		except AttributeError as e:
			pass #Omitted because logger is being built in contructor method

		return actor_ref

	def find_by_id(self, unique_id):
		try:
			return self.__actors_ref[unique_id]
		except KeyError as e:
			raise exceptions.ActorNotFound(unique_id)

	def get_parent(self, unique_id):
		return self.find_by_id(unique_id).parent()

	def get_child(self, child_unique_id, parent_unique_id):
		try:
			children = self.__actors_hierarchy[parent_unique_id]

			if child_unique_id in children:
				return self.__actors_ref[child_unique_id]
			else:
				raise exceptions.ActorNotFound(child_unique_id, 'Is {0} his actual parent?'.format(parent_unique_id))			

		except KeyError as e:
			raise exceptions.ActorNotFound(parent_unique_id)

	def is_child_of(unique_id, parent_unique_id):

		try:
			self.get_child(unique_id, parent_unique_id)
		except exceptions.ActorNotFound:
			return False

		return True


	def get_children(self, parent_unique_id):
		try:
			children = self.__actors_hierarchy[parent_unique_id]
			return [self.__actors_ref[child] for child in children]
		except KeyError as e:
			raise exceptions.ActorNotFound(parent_unique_id)

	def have_children(self, parent_unique_id):
		try:
			return len(self.get_children(parent_unique_id)) > 0
		except exceptions.ActorNotFound as e:
			return False

	def marooned_messages(self):
		return self.__dead_letters_manager

	def active_logger(self):
		return self.__syslogger


import exceptions

from longtang.actors.logger import logger, messages, factory


class SystemLoggerFactory:

	@staticmethod
	def createIfAbsent(system, level):
		syslogger_ref=None

		if level is not None:
			try:
				syslogger_ref = system.find_by_id('sys-logger')
			except exceptions.ActorNotFound as e:
				syslogger_ref = system.with_factory(factory.LoggerActorFactory(level),'sys-logger')

		return SystemLogger(syslogger_ref)


class SystemLogger:
	def __init__(self, logger_actor_ref):
		self.__logger_actor_ref=logger_actor_ref

	def info(self, actor_id, message, source_file):
		if self.__logger_actor_ref is not None and actor_id is not self.__logger_actor_ref.unique_id():
			self.__logger_actor_ref.tell(messages.FileInformationEventMessage(actor_id, source_file, message))

	def error(self, actor_id, message, source_file):
		if self.__logger_actor_ref is not None and actor_id is not self.__logger_actor_ref.unique_id():
			self.__logger_actor_ref.tell(messages.FileErrorEventMessage(actor_id, source_file, message))

	def debug(self, actor_id, message, source_file):
		if self.__logger_actor_ref is not None and actor_id is not self.__logger_actor_ref.unique_id():
			self.__logger_actor_ref.tell(messages.FileDebugEventMessage(actor_id, source_file, message))

	def warn(self, actor_id, message, source_file):
		if self.__logger_actor_ref is not None and actor_id is not self.__logger_actor_ref.unique_id():
			self.__logger_actor_ref.tell(messages.FileWarnEventMessage(actor_id, source_file, message))		


# -*- coding: utf-8 -*-

#http://docs.python.org/2/library/logging.html#formatter-objects

import messages
import logging
import sys

from longtang.actors import actors

class LoggerActor(actors.Actor):

	def __init__(self, level, system_ref, unique_id):
		actors.Actor.__init__(self, system_ref, unique_id)

		self.__level=level

	def pre_setup(self):

		global my_logger

		if not globals().has_key('my_logger'):
			# create logger
			root_logger = logging.getLogger()
			root_logger.handlers = []

			my_logger = logging.getLogger('logger-actor')
			#my_logger.setLevel(logging.DEBUG)
			my_logger.setLevel(self.__level)
			#my_logger.propagate = False

			# create formatter
			formatter = logging.Formatter('[%(levelname)s][%(asctime)s][%(actor)s](%(source_file)s) @=> %(message)s')

			# create console handler and set level to debug
			ch = logging.StreamHandler(sys.stdout)
			ch.setLevel(logging.DEBUG)
			ch.setFormatter(formatter)

			# add handler to logger
			my_logger.addHandler(ch)

	def receive(self, message):

		if isinstance(message, messages.FileEventMessage):

			extra_info = {'actor': message.actor(), 'source_file': message.source_file()}
			if message.type() is 'INFO':
				my_logger.info(message.message(), extra=extra_info)
			elif message.type() is 'ERROR':
				my_logger.error(message.message(), extra=extra_info)
			elif message.type() is 'DEBUG':
				my_logger.debug(message.message(), extra=extra_info)
			elif message.type() is 'WARN':
				my_logger.warning(message.message(), extra=extra_info)
		else:
			self.notify_marooned_message(message)


import messages
import tracking

from longtang.actors import domain as actor_domain

from longtang.common import interactions

class MediaTrackingFacade():

	def __init__(self, system_ref, mediatracking_actor_ref):
		self.__actor_ref=mediatracking_actor_ref
		self.__system_ref=system_ref

	@staticmethod
	def from_system(system_ref):
		actor_ref = system_ref.get_by_id('mediatracking-actor')
		return MediaTrackingFacade(system_ref, actor_ref)

	@staticmethod
	def from_context(context_ref):
		return MediaTrackingFacade.from_system(context_ref._system_ref())

	@staticmethod
	def create_within(system_ref):

		system_ref_tmp = system_ref

		#Actors domain
		if isinstance(system_ref, actor_domain.SystemRefWrapper):
			system_ref_tmp = system_ref._system_ref()
			actor_ref = system_ref.from_type(tracking.TrackingActor,'mediatracking-actor')
		else:
			actor_ref = system_ref_tmp.from_type(tracking.TrackingActor,'mediatracking-actor', None)

		return MediaTrackingFacade(system_ref_tmp, actor_ref)

	def create_tracking(self, mediapath):
		response = interactions.InteractionPatterns.ask(self.__system_ref,self.__actor_ref, messages.CreateTrackingEntry(mediapath))
		return response.tracking_id()

	def lookup(self, tracking_id):
		response = interactions.InteractionPatterns.ask(self.__system_ref,self.__actor_ref, messages.LookupTrackingEntry(tracking_id))
		return response.tracking_info()

	def notify(self, tracking_id, event):
		self.__actor_ref.tell(messages.RegisterTrackingEvent(tracking_id,event), None)

	def retrieve_summary(self):
		response = interactions.InteractionPatterns.ask(self.__system_ref,self.__actor_ref, messages.GenerateSummary())
		return response.summary()		


import gevent

from longtang.actors import actors, messages

class InteractionPatterns:

	@staticmethod
	def ask(system_ref, actor_ref, message):

		#TODO The name could be dynamic i think
		host_actor = system_ref.from_type(HostActor,'interactions-actor', None)
		actor_ref.tell(message,host_actor)

		#system.wait_for(host_actor)
		gevent.joinall([host_actor._instance()])

		#TODO WE should remove this actor from the registry
		return host_actor._instance().response()

class HostActor(actors.Actor):
	def __init__(self, system_ref, unique_id):
		actors.Actor.__init__(self, system_ref, unique_id)
		self.__response=[]

	def receive(self, message):
		self.__response.append(message)
		self._silently_end()

	def response(self):
		return self.__response[0]


#https://github.com/hamcrest/PyHamcrest
#http://packages.python.org/PyHamcrest/

import gevent
import unittest
import os

from hamcrest import *
from longtang.actors.tracking import tracking, messages
from longtang.system import system
from longtang.actors import testutils
from longtang.common import interactions, mediatracking

class TestMediaTrackingActor(unittest.TestCase):

	def test_creation(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')

		try:
			assert_that(actor_system.find_by_id('tracking-actor'), is_not(None), 'Tracking actor does not exist within system')
		except exceptions.ActorNotFound as e:
			self.fail(str(e))
						
		actor_system.shutdown()

	def test_add_single_tracking(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy.mp3'))

		assert_that(response, is_(not_none()),'Tracking identifier message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEntryCreated)),'Tracking response message is not of the right type')
		assert_that(response.tracking_id(), is_(not_none()),'Tracking identifier is missing')

		actor_system.shutdown()

	def test_retrieve_single_tracking(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy-2.mp3')).tracking_id()

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.LookupTrackingEntry(tracking_id))

		assert_that(response, is_(not_none()),'Tracking information message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEntryFound)),'Tracking information message is not of the right type')
		assert_that(response.tracking_info().mediafile.sourcepath, is_(equal_to('/dummy-2.mp3')),'Tracking identifier is missing')

		actor_system.shutdown()

	def test_retrieve_single_tracking_notfound(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy-2.mp3')).tracking_id()

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.LookupTrackingEntry('nonexistingid'))

		assert_that(response, is_(not_none()),'Tracking information message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEntryNotFound)),'Tracking information message is not of the right type')
		assert_that(response.tracking_id(), is_(equal_to('nonexistingid')),'Tracking identifier is missing')		

		actor_system.shutdown()

	def test_notify_non_altering_tracking_event(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy-non-tracking.mp3')).tracking_id()

		event = mediatracking.MediaFileTrackingEvent('NONALTERING', 'This is a non-altering event')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.RegisterTrackingEvent(tracking_id,event))

		assert_that(response, is_(not_none()),'Tracking information message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEventSuccessfullyRegistered)),'Tracking event response is not of the right type')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.LookupTrackingEntry(tracking_id))

		assert_that(response.tracking_info().tracking_events.of_type('NONALTERING').is_empty(), is_(False),'Tracking events of type NONALTERING are empty')
		assert_that(response.tracking_info().tracking_events.of_type('NONALTERING').get(0), is_(not_none()),'Tracked events does not exist')

		actor_system.shutdown()

	def test_notify_altering_tracking_event(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy-non-tracking.mp3')).tracking_id()

		event = DestinationUpdateTrackingEvent('DESTINATION', 'This is an altering event', '/target/path/dummy.mp3')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.RegisterTrackingEvent(tracking_id,event))

		assert_that(response, is_(not_none()),'Tracking information message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEventSuccessfullyRegistered)),'Tracking event response is not of the right type')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.LookupTrackingEntry(tracking_id))

		assert_that(response.tracking_info().tracking_events.of_type('DESTINATION').is_empty(), is_(False),'Tracking events of type NONALTERING are empty')
		assert_that(response.tracking_info().tracking_events.of_type('DESTINATION').get(0), is_(not_none()),'Tracked events does not exist')
		assert_that(response.tracking_info().mediafile.destinationpath, is_(equal_to('/target/path/dummy.mp3')),'Mediafile information has not been updated')

		actor_system.shutdown()
		
class DestinationUpdateTrackingEvent(mediatracking.MediaFileTrackingEvent):
	def __init__(self, entry_type, description, destination):
		mediatracking.MediaFileTrackingEvent.__init__(self, entry_type, description,{"destination": destination})

	def process(self, file_descriptor):
		file_descriptor.destinationpath=self.context_information()["destination"]
		return file_descriptor


import unittest
import os

from longtang.actors import testutils
from longtang.actors.filesystem import foldermanager, messages, domain
from longtang.common import domain as common_domain
from hamcrest import *


class TestFolderManagerActor(unittest.TestCase):

	@classmethod
	def setUpClass(cls):
		global base_test_dir
		base_test_dir = testutils.create_tmp_dir(suffix='folder_management_test')

	@classmethod
	def tearDownClass(cls):
		testutils.remove_tmp_dir(base_test_dir)

	def test_create_folder_from_metadata_message(self):
		
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = os.path.join(data_dir, 'sample.mp3')

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('Folder Artist').title('Folder Title').album('Folder Album').track_number('7').build()

		folder_tester = testutils.TestActorBuilder().withType(foldermanager.FolderManagerActor).build()

		folder_tester.tell(messages.CreateFolderFromMetadata(source_file, metadata, base_test_dir, 'tracking-id'))

		assert_that(folder_tester.inspector().num_instances(messages.FolderFromMetadataSuccessfullyCreated), is_(equal_to(1)), 'Total amount of FolderFromMetadataCreated messages received is wrong')		

		message = folder_tester.inspector().get_first(messages.FolderFromMetadataSuccessfullyCreated)

		assert_that(message.source_file(), is_(not_none()), 'Source file is empty')
		assert_that(message.metadata(), is_(not_none()), 'File metadata information is empty')
		assert_that(message.new_dir_path(), is_(not_none()), 'New dir path information is empty')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')

	def test_create_folder_from_metadata_folder_existance(self):
		
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = os.path.join(data_dir, 'sample.mp3')

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('Folder Artist').title('Folder Title').album('Folder Album').track_number('7').build()

		folder_tester = testutils.TestActorBuilder().withType(foldermanager.FolderManagerActor).build()

		folder_tester.tell(messages.CreateFolderFromMetadata(source_file, metadata, base_test_dir, 'tracking-id'))

		message = folder_tester.inspector().get_first(messages.FolderFromMetadataSuccessfullyCreated)

		full_dir_path= os.path.join(base_test_dir, 'Folder Artist', 'Folder Album')

		assert_that(os.path.isdir(full_dir_path), is_(equal_to(True)), 'Target directory does not exist or full name is not right')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')

	def test_folder_name_capitalization(self):
		
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = os.path.join(data_dir, 'sample.mp3')

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('CAPITAL ARTIST').title('CAPITAL TITLE').album('CAPITAL ALBUM').track_number('7').build()

		folder_tester = testutils.TestActorBuilder().withType(foldermanager.FolderManagerActor).build()

		folder_tester.tell(messages.CreateFolderFromMetadata(source_file, metadata, base_test_dir, 'tracking-id'))

		message = folder_tester.inspector().get_first(messages.FolderFromMetadataSuccessfullyCreated)

		full_dir_path= os.path.join(base_test_dir, 'Capital Artist', 'Capital Album')

		assert_that(os.path.isdir(full_dir_path), is_(equal_to(True)), 'Target directory does not exist or full name is not right')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')

	def test_create_folder_from_metadata_dir_failure(self):
		
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = os.path.join(data_dir, 'sample.mp3')

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('Folder Artist').title('Folder Title').album('Folder Album').track_number('7').build()

		folder_tester = testutils.TestActorBuilder().withType(foldermanager.FolderManagerActor).build()

		folder_tester.tell(messages.CreateFolderFromMetadata(source_file, metadata, '/dummy_dir', 'tracking-id'))

		assert_that(folder_tester.inspector().num_instances(messages.CreateFolderFromMetadataFailed), is_(equal_to(1)), 'Total amount of CreateFolderFromMetadataFailed messages received is wrong')		

		message = folder_tester.inspector().get_first(messages.CreateFolderFromMetadataFailed)

		assert_that(message.source_file(), is_(not_none()), 'Source file is empty')
		assert_that(message.metadata(), is_(not_none()), 'File metadata information is empty')
		assert_that(message.base_dir(), is_(not_none()), 'Base dir path information is empty')	
		assert_that(message.reason(), is_(not_none()), 'Reason information is empty')
		assert_that(message.source_message(), is_(not_none()), 'Source message information is empty')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')


#http://docs.python.org/2/howto/unicode

# -*- coding: utf-8 -*-

import fnmatch
import os
import messages
import types

from longtang.actors import actors

class FilePoller(actors.Actor):
	def receive(self, message):

		if isinstance(message, messages.StartPolling):

			source = message.source()

			if type(source) is not types.UnicodeType:
				source = unicode(message.source(), encoding='utf-8', errors='replace')

			self.logger().info(u'Polling request recieved. Polling source: {0}'.format(source))

			try:
				for root, dirnames, filenames in os.walk(source):
	  				for filename in fnmatch.filter(filenames, u'*.mp3'):

	  					full_path = os.path.join(root, filename)

						self.logger().info(u'New file found: {0}'.format(full_path))

						self.sender().tell(messages.AudioFileFound(full_path), self.myself())

				self.sender().tell(messages.FilePollingDone(), self.myself())
			except Exception as e:
				self.logger().warn(u'Polling process interrupted. Reason: {0}'.format(str(e)))
				self.sender().tell(messages.FilePollingDone(), self.myself())

		else: 
			self.notify_marooned_message(message)

